﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace MapViewer
{
    public partial class MapViewControl : PictureBox
    {
        private MapFetcher fetcher;
        private Roads roads;
        public Roads Roads { get { return roads; } set { roads = value; } }
        public MapFetcher MapFetcher { get { return fetcher; } set { fetcher = value; } }
        

        public int Zoom { get; set; }
        public Point Coords { get; set; }
        Size TileSize = new Size(256, 256);

        int tilesX = 3, tilesY = 3;
        Bitmap[,] tilesToDisplay = new Bitmap[3, 3];

        public MapViewControl()
        {
            InitializeComponent();
            this.MouseDown+=new MouseEventHandler(onMouseDown);
            this.MouseUp += new MouseEventHandler(onMouseUp);
            this.MouseMove += new MouseEventHandler(onMouseMove);
            this.Resize+=new EventHandler(onResize);
        }

        private void MapViewControl_Load(object sender, EventArgs e)
        {

        }

        public void LoadMap() 
        {
            if( fetcher != null )
            fetcher.Clear();
            for (int y = Coords.Y; y< Coords.Y+tilesY; y++)
            {
                for (int x = Coords.X; x < Coords.X + tilesX; x++) 
                {
                    l(x,y);
                }
            }
            Redraw();
        }

        private void l(int x, int y) 
        {
            MapDataRequest request = new MapDataRequest();
            request.x = x;
            request.y = y;
            request.z = Zoom;
            request.canvasCoords = new Point(x-Coords.X,y-Coords.Y);
            tilesToDisplay[x-Coords.X, y-Coords.Y] = fetcher.notLoaded;
            request.callback = delegate(MapDataRequest r, Bitmap b)
            {
                tilesToDisplay[x - Coords.X, y - Coords.Y] = b;
                Redraw();
            };
            fetcher.getSquare(request);
        }

        public void Redraw() 
        {
            if (this.InvokeRequired) {
                this.Invoke(new ThreadStart(Redraw));
                return;
            }

            if (this.BackgroundImage == null || this.BackgroundImage.Size.Width != tilesX * TileSize.Width || this.BackgroundImage.Size.Height != tilesY * TileSize.Height)
            {
                this.BackgroundImage = new Bitmap(tilesX * TileSize.Width, tilesY * TileSize.Height);
            }

            using (Graphics g = Graphics.FromImage(BackgroundImage)) 
            {
                g.Clear(Color.Black);
                for (int y = 0; y < tilesY; y++)
                {
                    for (int x = 0; x < tilesX; x++)
                    {
                        int xc = x * TileSize.Width;
                        int yc = y * TileSize.Height;
                        if( tilesToDisplay[x, y] != null )
                        g.DrawImage(tilesToDisplay[x, y], xc,yc,TileSize.Width,TileSize.Height);

                    }
                }
            }
            /*if (roads.loaded) 
            {
                foreach (Roads.Road road in roads.roads.Values) 
                {
                    foreach (Roads.Crossroad c in road.crossroads) 
                    {
                        double x = c.x / (9 - Zoom) / 2;
                        double y = c.y / (9 - Zoom) / 2;

                    }
                }
            }*/
            this.Invalidate();
        }

        bool mouseDown = false;

        public void onMouseDown(object sender, MouseEventArgs e)
        {
            mouseDown = true;
        }

        public void onMouseUp(object sender, MouseEventArgs e)
        {
            mouseDown = false;
        }

        public void onMouseMove(object sender, MouseEventArgs e)
        {
            /*if (!mouseDown) return;
            int offX = (int)e.X / TileSize.Width;
            int offY = (int)e.Y / TileSize.Height;
            if (Math.Abs(offX) >= 1) Coords = new Point(Coords.X + offX,Coords.Y);
            if (Math.Abs(offY) >= 1) Coords = new Point(Coords.X, Coords.Y+offY);
            if (Math.Abs(offX) >= 1 || Math.Abs(offY) >= 1) LoadMap();
            Redraw();*/
        }

        void onResize(object source, EventArgs e) 
        {
            /*tilesX = (int)Math.Ceiling(this.Width / TileSize.Width * 1.0); 
            tilesY = (int)Math.Ceiling(this.Height / TileSize.Height * 1.0);
            if (tilesX < 3) tilesX = 3;
            if (tilesY < 3) tilesY = 3;
            tilesToDisplay = new Bitmap[tilesX, tilesY];
            LoadMap();*/
        }
    }
}
